from .api_constants import InvokerUnits
from .client import (
    Category,
    Consultant,
    Course,
    ExpertCourse,
    General,
    Lesson,
    MasterCourse,
    Package,
    ResponseCodeError,
    TopBanner,
)

__all__ = [
    "Category",
    "Consultant",
    "Course",
    "General",
    "Lesson",
    "MasterCourse",
    "ExpertCourse",
    "Package",
    "ResponseCodeError",
    "TopBanner",
    "InvokerUnits",
]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: course_lesson/grpc/course_lesson.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n&course_lesson/grpc/course_lesson.proto\x12\x15\x64\x61rsman.course_lesson\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1cgoogle/protobuf/struct.proto\"\xa7\x01\n\x11\x41\x64\x64\x43ommentRequest\x12\x1d\n\x10owner_identifier\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\t\x12\x17\n\x0fuser_identifier\x18\x04 \x01(\t\x12\x17\n\nowner_slug\x18\x05 \x01(\tH\x01\x88\x01\x01\x42\x13\n\x11_owner_identifierB\r\n\x0b_owner_slug\"y\n\x12\x41\x64\x64\x43ommentResponse\x12\x14\n\x07message\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x11\n\x04\x63ode\x18\x02 \x01(\tH\x01\x88\x01\x01\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.StructB\n\n\x08_messageB\x07\n\x05_code\"w\n\x10\x43\x61tegoryResponse\x12\x14\n\x07message\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x11\n\x04\x63ode\x18\x02 \x01(\tH\x01\x88\x01\x01\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.StructB\n\n\x08_messageB\x07\n\x05_code\"g\n\x17\x43\x61tegoryRetrieveRequest\x12\x12\n\nidentifier\x18\x01 \x01(\t\x12\x0c\n\x04slug\x18\x02 \x01(\t\x12\x19\n\x0cinvoker_unit\x18\x03 \x01(\tH\x00\x88\x01\x01\x42\x0f\n\r_invoker_unit\"\xe7\x01\n\x11\x43onsultantRequest\x12\x16\n\tfull_name\x18\x02 \x01(\tH\x00\x88\x01\x01\x12\x14\n\x0cphone_number\x18\x03 \x01(\t\x12\x16\n\titem_type\x18\x04 \x01(\tH\x01\x88\x01\x01\x12\x1c\n\x0fitem_identifier\x18\x05 \x01(\tH\x02\x88\x01\x01\x12#\n\x16preferred_contact_time\x18\x06 \x01(\tH\x03\x88\x01\x01\x42\x0c\n\n_full_nameB\x0c\n\n_item_typeB\x12\n\x10_item_identifierB\x19\n\x17_preferred_contact_time\"y\n\x12\x43onsultantResponse\x12\x14\n\x07message\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x11\n\x04\x63ode\x18\x02 \x01(\tH\x01\x88\x01\x01\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.StructB\n\n\x08_messageB\x07\n\x05_code\"[\n\x19\x43ourseBulkRetrieveRequest\x12\x12\n\nidentifier\x18\x01 \x03(\t\x12\x19\n\x0cinvoker_unit\x18\x02 \x01(\tH\x00\x88\x01\x01\x42\x0f\n\r_invoker_unit\"\x9b\x01\n\x11\x43ourseListRequest\x12\x15\n\x08order_by\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x13\n\x0bpage_number\x18\x02 \x01(\x05\x12\x11\n\tpage_size\x18\x03 \x01(\x05\x12\x16\n\titem_type\x18\x04 \x01(\tH\x01\x88\x01\x01\x12\x14\n\x0cinvoker_unit\x18\x05 \x01(\tB\x0b\n\t_order_byB\x0c\n\n_item_type\"u\n\x0e\x43ourseResponse\x12\x14\n\x07message\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x11\n\x04\x63ode\x18\x02 \x01(\tH\x01\x88\x01\x01\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.StructB\n\n\x08_messageB\x07\n\x05_code\"m\n\x1d\x43ourseRetrieveCommentsRequest\x12\x12\n\nidentifier\x18\x01 \x01(\t\x12\x0c\n\x04slug\x18\x02 \x01(\t\x12\x19\n\x0cinvoker_unit\x18\x03 \x01(\tH\x00\x88\x01\x01\x42\x0f\n\r_invoker_unit\"j\n\x1a\x43ourseRetrieveRatesRequest\x12\x12\n\nidentifier\x18\x01 \x01(\t\x12\x0c\n\x04slug\x18\x02 \x01(\t\x12\x19\n\x0cinvoker_unit\x18\x03 \x01(\tH\x00\x88\x01\x01\x42\x0f\n\r_invoker_unit\"e\n\x15\x43ourseRetrieveRequest\x12\x12\n\nidentifier\x18\x01 \x01(\t\x12\x0c\n\x04slug\x18\x02 \x01(\t\x12\x19\n\x0cinvoker_unit\x18\x03 \x01(\tH\x00\x88\x01\x01\x42\x0f\n\r_invoker_unit\"v\n\x0fGeneralResponse\x12\x14\n\x07message\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x11\n\x04\x63ode\x18\x02 \x01(\tH\x01\x88\x01\x01\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.StructB\n\n\x08_messageB\x07\n\x05_code\"B\n\x16GeneralRetrieveRequest\x12\x12\n\nidentifier\x18\x01 \x01(\t\x12\x14\n\x0cinvoker_unit\x18\x02 \x01(\t\",\n\x16InstallmentTermRequest\x12\x12\n\nidentifier\x18\x01 \x01(\t\"[\n\x19LessonBulkRetrieveRequest\x12\x12\n\nidentifier\x18\x01 \x03(\t\x12\x19\n\x0cinvoker_unit\x18\x02 \x01(\tH\x00\x88\x01\x01\x42\x0f\n\r_invoker_unit\"\x8b\x01\n\x11LessonListRequest\x12\x15\n\x08order_by\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x13\n\x0bpage_number\x18\x02 \x01(\x05\x12\x11\n\tpage_size\x18\x03 \x01(\x05\x12\x19\n\x0cinvoker_unit\x18\x04 \x01(\tH\x01\x88\x01\x01\x42\x0b\n\t_order_byB\x0f\n\r_invoker_unit\"u\n\x0eLessonResponse\x12\x14\n\x07message\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x11\n\x04\x63ode\x18\x02 \x01(\tH\x01\x88\x01\x01\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.StructB\n\n\x08_messageB\x07\n\x05_code\"m\n\x1dLessonRetrieveCommentsRequest\x12\x12\n\nidentifier\x18\x01 \x01(\t\x12\x0c\n\x04slug\x18\x02 \x01(\t\x12\x19\n\x0cinvoker_unit\x18\x03 \x01(\tH\x00\x88\x01\x01\x42\x0f\n\r_invoker_unit\"e\n\x15LessonRetrieveRequest\x12\x12\n\nidentifier\x18\x01 \x01(\t\x12\x0c\n\x04slug\x18\x02 \x01(\t\x12\x19\n\x0cinvoker_unit\x18\x03 \x01(\tH\x00\x88\x01\x01\x42\x0f\n\r_invoker_unit\"\x93\x01\n\x1fMasterCourseNotificationRequest\x12\x12\n\nfirst_name\x18\x01 \x01(\t\x12\x11\n\tlast_name\x18\x02 \x01(\t\x12\x14\n\x0cphone_number\x18\x03 \x01(\t\x12\x15\n\rmaster_course\x18\x04 \x01(\t\x12\x12\n\x05state\x18\x05 \x01(\x03H\x00\x88\x01\x01\x42\x08\n\x06_state\"T\n\"MasterCoursePreRegistrationRequest\x12\x15\n\rmaster_course\x18\x01 \x01(\t\x12\x17\n\x0fuser_identifier\x18\x02 \x01(\t\".\n\x18MasterCourseStateRequest\x12\x12\n\nidentifier\x18\x01 \x01(\t\"*\n\rMasterRequest\x12\x19\n\x11master_identifier\x18\x01 \x01(\t\"u\n\x0eMasterResponse\x12\x14\n\x07message\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x11\n\x04\x63ode\x18\x02 \x01(\tH\x01\x88\x01\x01\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.StructB\n\n\x08_messageB\x07\n\x05_code\"\x99\x01\n\x14NotificationResponse\x12\x14\n\x07message\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x11\n\x04\x63ode\x18\x02 \x01(\tH\x01\x88\x01\x01\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.Struct\x12\x12\n\x05state\x18\x04 \x01(\x03H\x02\x88\x01\x01\x42\n\n\x08_messageB\x07\n\x05_codeB\x08\n\x06_state\"\\\n\x1aPackageBulkRetrieveRequest\x12\x12\n\nidentifier\x18\x01 \x03(\t\x12\x19\n\x0cinvoker_unit\x18\x02 \x01(\tH\x00\x88\x01\x01\x42\x0f\n\r_invoker_unit\"\x8c\x01\n\x12PackageListRequest\x12\x15\n\x08order_by\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x13\n\x0bpage_number\x18\x02 \x01(\x05\x12\x11\n\tpage_size\x18\x03 \x01(\x05\x12\x19\n\x0cinvoker_unit\x18\x04 \x01(\tH\x01\x88\x01\x01\x42\x0b\n\t_order_byB\x0f\n\r_invoker_unit\"v\n\x0fPackageResponse\x12\x14\n\x07message\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x11\n\x04\x63ode\x18\x02 \x01(\tH\x01\x88\x01\x01\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.StructB\n\n\x08_messageB\x07\n\x05_code\"f\n\x16PackageRetrieveRequest\x12\x12\n\nidentifier\x18\x01 \x01(\t\x12\x0c\n\x04slug\x18\x02 \x01(\t\x12\x19\n\x0cinvoker_unit\x18\x03 \x01(\tH\x00\x88\x01\x01\x42\x0f\n\r_invoker_unit\"~\n\x17PreRegistrationResponse\x12\x14\n\x07message\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x11\n\x04\x63ode\x18\x02 \x01(\tH\x01\x88\x01\x01\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.StructB\n\n\x08_messageB\x07\n\x05_code\"\x90\x01\n\x0bRateRequest\x12\x1d\n\x10owner_identifier\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x17\n\x0fuser_identifier\x18\x02 \x01(\t\x12\x0c\n\x04rate\x18\x03 \x01(\x02\x12\x17\n\nowner_slug\x18\x04 \x01(\tH\x01\x88\x01\x01\x42\x13\n\x11_owner_identifierB\r\n\x0b_owner_slug\"s\n\x0cRateResponse\x12\x14\n\x07message\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x11\n\x04\x63ode\x18\x02 \x01(\tH\x01\x88\x01\x01\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.StructB\n\n\x08_messageB\x07\n\x05_code\"o\n\x08Response\x12\x14\n\x07message\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x11\n\x04\x63ode\x18\x02 \x01(\tH\x01\x88\x01\x01\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.StructB\n\n\x08_messageB\x07\n\x05_code\"t\n\rStateResponse\x12\x14\n\x07message\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x11\n\x04\x63ode\x18\x02 \x01(\tH\x01\x88\x01\x01\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.StructB\n\n\x08_messageB\x07\n\x05_code\"x\n\x11TopBannerResponse\x12\x14\n\x07message\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x11\n\x04\x63ode\x18\x02 \x01(\tH\x01\x88\x01\x01\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.StructB\n\n\x08_messageB\x07\n\x05_code\"p\n\x12TriggerLikeRequest\x12\x1d\n\x10owner_identifier\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x17\n\nowner_slug\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x13\n\x11_owner_identifierB\r\n\x0b_owner_slug\"z\n\x13TriggerLikeResponse\x12\x14\n\x07message\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x11\n\x04\x63ode\x18\x02 \x01(\tH\x01\x88\x01\x01\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.StructB\n\n\x08_messageB\x07\n\x05_code2\xd2\x01\n\x12\x43\x61tegoryController\x12U\n\x10HierarchicalList\x12\x16.google.protobuf.Empty\x1a\'.darsman.course_lesson.CategoryResponse\"\x00\x12\x65\n\x08Retrieve\x12..darsman.course_lesson.CategoryRetrieveRequest\x1a\'.darsman.course_lesson.CategoryResponse\"\x00\x32w\n\x14\x43onsultantController\x12_\n\x06\x43reate\x12(.darsman.course_lesson.ConsultantRequest\x1a).darsman.course_lesson.ConsultantResponse\"\x00\x32\xf3\x07\n\x10\x43ourseController\x12\x63\n\x0c\x42ulkRetrieve\x12\x30.darsman.course_lesson.CourseBulkRetrieveRequest\x1a\x1f.darsman.course_lesson.Response\"\x00\x12`\n\x07\x43omment\x12(.darsman.course_lesson.AddCommentRequest\x1a).darsman.course_lesson.AddCommentResponse\"\x00\x12\x63\n\x12\x46\x65tchMasterCourses\x12$.darsman.course_lesson.MasterRequest\x1a%.darsman.course_lesson.MasterResponse\"\x00\x12_\n\x04Like\x12).darsman.course_lesson.TriggerLikeRequest\x1a*.darsman.course_lesson.TriggerLikeResponse\"\x00\x12Y\n\x04List\x12(.darsman.course_lesson.CourseListRequest\x1a%.darsman.course_lesson.CourseResponse\"\x00\x12Q\n\x04Rate\x12\".darsman.course_lesson.RateRequest\x1a#.darsman.course_lesson.RateResponse\"\x00\x12\x61\n\x08Retrieve\x12,.darsman.course_lesson.CourseRetrieveRequest\x1a%.darsman.course_lesson.CourseResponse\"\x00\x12q\n\x10RetrieveComments\x12\x34.darsman.course_lesson.CourseRetrieveCommentsRequest\x1a%.darsman.course_lesson.CourseResponse\"\x00\x12k\n\rRetrieveRates\x12\x31.darsman.course_lesson.CourseRetrieveRatesRequest\x1a%.darsman.course_lesson.CourseResponse\"\x00\x12\x61\n\x06UnLike\x12).darsman.course_lesson.TriggerLikeRequest\x1a*.darsman.course_lesson.TriggerLikeResponse\"\x00\x32\xf9\x07\n\x16\x45xpertCourseController\x12\x63\n\x0c\x42ulkRetrieve\x12\x30.darsman.course_lesson.CourseBulkRetrieveRequest\x1a\x1f.darsman.course_lesson.Response\"\x00\x12`\n\x07\x43omment\x12(.darsman.course_lesson.AddCommentRequest\x1a).darsman.course_lesson.AddCommentResponse\"\x00\x12\x63\n\x12\x46\x65tchMasterCourses\x12$.darsman.course_lesson.MasterRequest\x1a%.darsman.course_lesson.MasterResponse\"\x00\x12_\n\x04Like\x12).darsman.course_lesson.TriggerLikeRequest\x1a*.darsman.course_lesson.TriggerLikeResponse\"\x00\x12Y\n\x04List\x12(.darsman.course_lesson.CourseListRequest\x1a%.darsman.course_lesson.CourseResponse\"\x00\x12Q\n\x04Rate\x12\".darsman.course_lesson.RateRequest\x1a#.darsman.course_lesson.RateResponse\"\x00\x12\x61\n\x08Retrieve\x12,.darsman.course_lesson.CourseRetrieveRequest\x1a%.darsman.course_lesson.CourseResponse\"\x00\x12q\n\x10RetrieveComments\x12\x34.darsman.course_lesson.CourseRetrieveCommentsRequest\x1a%.darsman.course_lesson.CourseResponse\"\x00\x12k\n\rRetrieveRates\x12\x31.darsman.course_lesson.CourseRetrieveRatesRequest\x1a%.darsman.course_lesson.CourseResponse\"\x00\x12\x61\n\x06UnLike\x12).darsman.course_lesson.TriggerLikeRequest\x1a*.darsman.course_lesson.TriggerLikeResponse\"\x00\x32x\n\x11GeneralController\x12\x63\n\x08Retrieve\x12-.darsman.course_lesson.GeneralRetrieveRequest\x1a&.darsman.course_lesson.GeneralResponse\"\x00\x32\x8a\x04\n\x10LessonController\x12\x63\n\x0c\x42ulkRetrieve\x12\x30.darsman.course_lesson.LessonBulkRetrieveRequest\x1a\x1f.darsman.course_lesson.Response\"\x00\x12`\n\x07\x43omment\x12(.darsman.course_lesson.AddCommentRequest\x1a).darsman.course_lesson.AddCommentResponse\"\x00\x12Y\n\x04List\x12(.darsman.course_lesson.LessonListRequest\x1a%.darsman.course_lesson.LessonResponse\"\x00\x12\x61\n\x08Retrieve\x12,.darsman.course_lesson.LessonRetrieveRequest\x1a%.darsman.course_lesson.LessonResponse\"\x00\x12q\n\x10RetrieveComments\x12\x34.darsman.course_lesson.LessonRetrieveCommentsRequest\x1a%.darsman.course_lesson.LessonResponse\"\x00\x32\xd4\x0b\n\x16MasterCourseController\x12\x63\n\x0c\x42ulkRetrieve\x12\x30.darsman.course_lesson.CourseBulkRetrieveRequest\x1a\x1f.darsman.course_lesson.Response\"\x00\x12`\n\x07\x43omment\x12(.darsman.course_lesson.AddCommentRequest\x1a).darsman.course_lesson.AddCommentResponse\"\x00\x12{\n\x12\x43reateNotification\x12\x36.darsman.course_lesson.MasterCourseNotificationRequest\x1a+.darsman.course_lesson.NotificationResponse\"\x00\x12\x84\x01\n\x15\x43reatePreRegistration\x12\x39.darsman.course_lesson.MasterCoursePreRegistrationRequest\x1a..darsman.course_lesson.PreRegistrationResponse\"\x00\x12\x63\n\x12\x46\x65tchMasterCourses\x12$.darsman.course_lesson.MasterRequest\x1a%.darsman.course_lesson.MasterResponse\"\x00\x12_\n\x04Like\x12).darsman.course_lesson.TriggerLikeRequest\x1a*.darsman.course_lesson.TriggerLikeResponse\"\x00\x12Y\n\x04List\x12(.darsman.course_lesson.CourseListRequest\x1a%.darsman.course_lesson.CourseResponse\"\x00\x12Q\n\x04Rate\x12\".darsman.course_lesson.RateRequest\x1a#.darsman.course_lesson.RateResponse\"\x00\x12\x61\n\x08Retrieve\x12,.darsman.course_lesson.CourseRetrieveRequest\x1a%.darsman.course_lesson.CourseResponse\"\x00\x12q\n\x10RetrieveComments\x12\x34.darsman.course_lesson.CourseRetrieveCommentsRequest\x1a%.darsman.course_lesson.CourseResponse\"\x00\x12k\n\x17RetrieveInstallmentTerm\x12-.darsman.course_lesson.InstallmentTermRequest\x1a\x1f.darsman.course_lesson.Response\"\x00\x12k\n\rRetrieveRates\x12\x31.darsman.course_lesson.CourseRetrieveRatesRequest\x1a%.darsman.course_lesson.CourseResponse\"\x00\x12h\n\rRetrieveState\x12/.darsman.course_lesson.MasterCourseStateRequest\x1a$.darsman.course_lesson.StateResponse\"\x00\x12\x61\n\x06UnLike\x12).darsman.course_lesson.TriggerLikeRequest\x1a*.darsman.course_lesson.TriggerLikeResponse\"\x00\x32\xbb\x02\n\x11PackageController\x12\x64\n\x0c\x42ulkRetrieve\x12\x31.darsman.course_lesson.PackageBulkRetrieveRequest\x1a\x1f.darsman.course_lesson.Response\"\x00\x12[\n\x04List\x12).darsman.course_lesson.PackageListRequest\x1a&.darsman.course_lesson.PackageResponse\"\x00\x12\x63\n\x08Retrieve\x12-.darsman.course_lesson.PackageRetrieveRequest\x1a&.darsman.course_lesson.PackageResponse\"\x00\x32l\n\x13TopBannerController\x12U\n\x0fRetrieveCurrent\x12\x16.google.protobuf.Empty\x1a(.darsman.course_lesson.TopBannerResponse\"\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'course_lesson.grpc.course_lesson_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_ADDCOMMENTREQUEST']._serialized_start=125
  _globals['_ADDCOMMENTREQUEST']._serialized_end=292
  _globals['_ADDCOMMENTRESPONSE']._serialized_start=294
  _globals['_ADDCOMMENTRESPONSE']._serialized_end=415
  _globals['_CATEGORYRESPONSE']._serialized_start=417
  _globals['_CATEGORYRESPONSE']._serialized_end=536
  _globals['_CATEGORYRETRIEVEREQUEST']._serialized_start=538
  _globals['_CATEGORYRETRIEVEREQUEST']._serialized_end=641
  _globals['_CONSULTANTREQUEST']._serialized_start=644
  _globals['_CONSULTANTREQUEST']._serialized_end=875
  _globals['_CONSULTANTRESPONSE']._serialized_start=877
  _globals['_CONSULTANTRESPONSE']._serialized_end=998
  _globals['_COURSEBULKRETRIEVEREQUEST']._serialized_start=1000
  _globals['_COURSEBULKRETRIEVEREQUEST']._serialized_end=1091
  _globals['_COURSELISTREQUEST']._serialized_start=1094
  _globals['_COURSELISTREQUEST']._serialized_end=1249
  _globals['_COURSERESPONSE']._serialized_start=1251
  _globals['_COURSERESPONSE']._serialized_end=1368
  _globals['_COURSERETRIEVECOMMENTSREQUEST']._serialized_start=1370
  _globals['_COURSERETRIEVECOMMENTSREQUEST']._serialized_end=1479
  _globals['_COURSERETRIEVERATESREQUEST']._serialized_start=1481
  _globals['_COURSERETRIEVERATESREQUEST']._serialized_end=1587
  _globals['_COURSERETRIEVEREQUEST']._serialized_start=1589
  _globals['_COURSERETRIEVEREQUEST']._serialized_end=1690
  _globals['_GENERALRESPONSE']._serialized_start=1692
  _globals['_GENERALRESPONSE']._serialized_end=1810
  _globals['_GENERALRETRIEVEREQUEST']._serialized_start=1812
  _globals['_GENERALRETRIEVEREQUEST']._serialized_end=1878
  _globals['_INSTALLMENTTERMREQUEST']._serialized_start=1880
  _globals['_INSTALLMENTTERMREQUEST']._serialized_end=1924
  _globals['_LESSONBULKRETRIEVEREQUEST']._serialized_start=1926
  _globals['_LESSONBULKRETRIEVEREQUEST']._serialized_end=2017
  _globals['_LESSONLISTREQUEST']._serialized_start=2020
  _globals['_LESSONLISTREQUEST']._serialized_end=2159
  _globals['_LESSONRESPONSE']._serialized_start=2161
  _globals['_LESSONRESPONSE']._serialized_end=2278
  _globals['_LESSONRETRIEVECOMMENTSREQUEST']._serialized_start=2280
  _globals['_LESSONRETRIEVECOMMENTSREQUEST']._serialized_end=2389
  _globals['_LESSONRETRIEVEREQUEST']._serialized_start=2391
  _globals['_LESSONRETRIEVEREQUEST']._serialized_end=2492
  _globals['_MASTERCOURSENOTIFICATIONREQUEST']._serialized_start=2495
  _globals['_MASTERCOURSENOTIFICATIONREQUEST']._serialized_end=2642
  _globals['_MASTERCOURSEPREREGISTRATIONREQUEST']._serialized_start=2644
  _globals['_MASTERCOURSEPREREGISTRATIONREQUEST']._serialized_end=2728
  _globals['_MASTERCOURSESTATEREQUEST']._serialized_start=2730
  _globals['_MASTERCOURSESTATEREQUEST']._serialized_end=2776
  _globals['_MASTERREQUEST']._serialized_start=2778
  _globals['_MASTERREQUEST']._serialized_end=2820
  _globals['_MASTERRESPONSE']._serialized_start=2822
  _globals['_MASTERRESPONSE']._serialized_end=2939
  _globals['_NOTIFICATIONRESPONSE']._serialized_start=2942
  _globals['_NOTIFICATIONRESPONSE']._serialized_end=3095
  _globals['_PACKAGEBULKRETRIEVEREQUEST']._serialized_start=3097
  _globals['_PACKAGEBULKRETRIEVEREQUEST']._serialized_end=3189
  _globals['_PACKAGELISTREQUEST']._serialized_start=3192
  _globals['_PACKAGELISTREQUEST']._serialized_end=3332
  _globals['_PACKAGERESPONSE']._serialized_start=3334
  _globals['_PACKAGERESPONSE']._serialized_end=3452
  _globals['_PACKAGERETRIEVEREQUEST']._serialized_start=3454
  _globals['_PACKAGERETRIEVEREQUEST']._serialized_end=3556
  _globals['_PREREGISTRATIONRESPONSE']._serialized_start=3558
  _globals['_PREREGISTRATIONRESPONSE']._serialized_end=3684
  _globals['_RATEREQUEST']._serialized_start=3687
  _globals['_RATEREQUEST']._serialized_end=3831
  _globals['_RATERESPONSE']._serialized_start=3833
  _globals['_RATERESPONSE']._serialized_end=3948
  _globals['_RESPONSE']._serialized_start=3950
  _globals['_RESPONSE']._serialized_end=4061
  _globals['_STATERESPONSE']._serialized_start=4063
  _globals['_STATERESPONSE']._serialized_end=4179
  _globals['_TOPBANNERRESPONSE']._serialized_start=4181
  _globals['_TOPBANNERRESPONSE']._serialized_end=4301
  _globals['_TRIGGERLIKEREQUEST']._serialized_start=4303
  _globals['_TRIGGERLIKEREQUEST']._serialized_end=4415
  _globals['_TRIGGERLIKERESPONSE']._serialized_start=4417
  _globals['_TRIGGERLIKERESPONSE']._serialized_end=4539
  _globals['_CATEGORYCONTROLLER']._serialized_start=4542
  _globals['_CATEGORYCONTROLLER']._serialized_end=4752
  _globals['_CONSULTANTCONTROLLER']._serialized_start=4754
  _globals['_CONSULTANTCONTROLLER']._serialized_end=4873
  _globals['_COURSECONTROLLER']._serialized_start=4876
  _globals['_COURSECONTROLLER']._serialized_end=5887
  _globals['_EXPERTCOURSECONTROLLER']._serialized_start=5890
  _globals['_EXPERTCOURSECONTROLLER']._serialized_end=6907
  _globals['_GENERALCONTROLLER']._serialized_start=6909
  _globals['_GENERALCONTROLLER']._serialized_end=7029
  _globals['_LESSONCONTROLLER']._serialized_start=7032
  _globals['_LESSONCONTROLLER']._serialized_end=7554
  _globals['_MASTERCOURSECONTROLLER']._serialized_start=7557
  _globals['_MASTERCOURSECONTROLLER']._serialized_end=9049
  _globals['_PACKAGECONTROLLER']._serialized_start=9052
  _globals['_PACKAGECONTROLLER']._serialized_end=9367
  _globals['_TOPBANNERCONTROLLER']._serialized_start=9369
  _globals['_TOPBANNERCONTROLLER']._serialized_end=9477
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import course_lesson_pb2 as course__lesson_dot_grpc_dot_course__lesson__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2


class CategoryControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.HierarchicalList = channel.unary_unary(
                '/darsman.course_lesson.CategoryController/HierarchicalList',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CategoryResponse.FromString,
                )
        self.Retrieve = channel.unary_unary(
                '/darsman.course_lesson.CategoryController/Retrieve',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CategoryRetrieveRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CategoryResponse.FromString,
                )


class CategoryControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def HierarchicalList(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Retrieve(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_CategoryControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'HierarchicalList': grpc.unary_unary_rpc_method_handler(
                    servicer.HierarchicalList,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CategoryResponse.SerializeToString,
            ),
            'Retrieve': grpc.unary_unary_rpc_method_handler(
                    servicer.Retrieve,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CategoryRetrieveRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CategoryResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'darsman.course_lesson.CategoryController', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class CategoryController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def HierarchicalList(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.CategoryController/HierarchicalList',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.CategoryResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Retrieve(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.CategoryController/Retrieve',
            course__lesson_dot_grpc_dot_course__lesson__pb2.CategoryRetrieveRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.CategoryResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)


class ConsultantControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Create = channel.unary_unary(
                '/darsman.course_lesson.ConsultantController/Create',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.ConsultantRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.ConsultantResponse.FromString,
                )


class ConsultantControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Create(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ConsultantControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Create': grpc.unary_unary_rpc_method_handler(
                    servicer.Create,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.ConsultantRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.ConsultantResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'darsman.course_lesson.ConsultantController', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class ConsultantController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Create(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.ConsultantController/Create',
            course__lesson_dot_grpc_dot_course__lesson__pb2.ConsultantRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.ConsultantResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)


class CourseControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.BulkRetrieve = channel.unary_unary(
                '/darsman.course_lesson.CourseController/BulkRetrieve',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseBulkRetrieveRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.Response.FromString,
                )
        self.Comment = channel.unary_unary(
                '/darsman.course_lesson.CourseController/Comment',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentResponse.FromString,
                )
        self.FetchMasterCourses = channel.unary_unary(
                '/darsman.course_lesson.CourseController/FetchMasterCourses',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.MasterRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.MasterResponse.FromString,
                )
        self.Like = channel.unary_unary(
                '/darsman.course_lesson.CourseController/Like',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeResponse.FromString,
                )
        self.List = channel.unary_unary(
                '/darsman.course_lesson.CourseController/List',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseListRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
                )
        self.Rate = channel.unary_unary(
                '/darsman.course_lesson.CourseController/Rate',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.RateRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.RateResponse.FromString,
                )
        self.Retrieve = channel.unary_unary(
                '/darsman.course_lesson.CourseController/Retrieve',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
                )
        self.RetrieveComments = channel.unary_unary(
                '/darsman.course_lesson.CourseController/RetrieveComments',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveCommentsRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
                )
        self.RetrieveRates = channel.unary_unary(
                '/darsman.course_lesson.CourseController/RetrieveRates',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveRatesRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
                )
        self.UnLike = channel.unary_unary(
                '/darsman.course_lesson.CourseController/UnLike',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeResponse.FromString,
                )


class CourseControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def BulkRetrieve(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Comment(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def FetchMasterCourses(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Like(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def List(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Rate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Retrieve(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RetrieveComments(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RetrieveRates(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UnLike(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_CourseControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'BulkRetrieve': grpc.unary_unary_rpc_method_handler(
                    servicer.BulkRetrieve,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseBulkRetrieveRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.Response.SerializeToString,
            ),
            'Comment': grpc.unary_unary_rpc_method_handler(
                    servicer.Comment,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentResponse.SerializeToString,
            ),
            'FetchMasterCourses': grpc.unary_unary_rpc_method_handler(
                    servicer.FetchMasterCourses,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.MasterRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.MasterResponse.SerializeToString,
            ),
            'Like': grpc.unary_unary_rpc_method_handler(
                    servicer.Like,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeResponse.SerializeToString,
            ),
            'List': grpc.unary_unary_rpc_method_handler(
                    servicer.List,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseListRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.SerializeToString,
            ),
            'Rate': grpc.unary_unary_rpc_method_handler(
                    servicer.Rate,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.RateRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.RateResponse.SerializeToString,
            ),
            'Retrieve': grpc.unary_unary_rpc_method_handler(
                    servicer.Retrieve,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.SerializeToString,
            ),
            'RetrieveComments': grpc.unary_unary_rpc_method_handler(
                    servicer.RetrieveComments,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveCommentsRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.SerializeToString,
            ),
            'RetrieveRates': grpc.unary_unary_rpc_method_handler(
                    servicer.RetrieveRates,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveRatesRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.SerializeToString,
            ),
            'UnLike': grpc.unary_unary_rpc_method_handler(
                    servicer.UnLike,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'darsman.course_lesson.CourseController', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class CourseController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def BulkRetrieve(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.CourseController/BulkRetrieve',
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseBulkRetrieveRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Comment(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.CourseController/Comment',
            course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def FetchMasterCourses(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.CourseController/FetchMasterCourses',
            course__lesson_dot_grpc_dot_course__lesson__pb2.MasterRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.MasterResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Like(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.CourseController/Like',
            course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def List(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.CourseController/List',
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseListRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Rate(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.CourseController/Rate',
            course__lesson_dot_grpc_dot_course__lesson__pb2.RateRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.RateResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Retrieve(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.CourseController/Retrieve',
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RetrieveComments(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.CourseController/RetrieveComments',
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveCommentsRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RetrieveRates(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.CourseController/RetrieveRates',
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveRatesRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UnLike(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.CourseController/UnLike',
            course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)


class ExpertCourseControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.BulkRetrieve = channel.unary_unary(
                '/darsman.course_lesson.ExpertCourseController/BulkRetrieve',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseBulkRetrieveRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.Response.FromString,
                )
        self.Comment = channel.unary_unary(
                '/darsman.course_lesson.ExpertCourseController/Comment',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentResponse.FromString,
                )
        self.FetchMasterCourses = channel.unary_unary(
                '/darsman.course_lesson.ExpertCourseController/FetchMasterCourses',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.MasterRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.MasterResponse.FromString,
                )
        self.Like = channel.unary_unary(
                '/darsman.course_lesson.ExpertCourseController/Like',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeResponse.FromString,
                )
        self.List = channel.unary_unary(
                '/darsman.course_lesson.ExpertCourseController/List',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseListRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
                )
        self.Rate = channel.unary_unary(
                '/darsman.course_lesson.ExpertCourseController/Rate',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.RateRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.RateResponse.FromString,
                )
        self.Retrieve = channel.unary_unary(
                '/darsman.course_lesson.ExpertCourseController/Retrieve',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
                )
        self.RetrieveComments = channel.unary_unary(
                '/darsman.course_lesson.ExpertCourseController/RetrieveComments',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveCommentsRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
                )
        self.RetrieveRates = channel.unary_unary(
                '/darsman.course_lesson.ExpertCourseController/RetrieveRates',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveRatesRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
                )
        self.UnLike = channel.unary_unary(
                '/darsman.course_lesson.ExpertCourseController/UnLike',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeResponse.FromString,
                )


class ExpertCourseControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def BulkRetrieve(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Comment(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def FetchMasterCourses(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Like(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def List(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Rate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Retrieve(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RetrieveComments(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RetrieveRates(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UnLike(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ExpertCourseControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'BulkRetrieve': grpc.unary_unary_rpc_method_handler(
                    servicer.BulkRetrieve,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseBulkRetrieveRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.Response.SerializeToString,
            ),
            'Comment': grpc.unary_unary_rpc_method_handler(
                    servicer.Comment,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentResponse.SerializeToString,
            ),
            'FetchMasterCourses': grpc.unary_unary_rpc_method_handler(
                    servicer.FetchMasterCourses,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.MasterRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.MasterResponse.SerializeToString,
            ),
            'Like': grpc.unary_unary_rpc_method_handler(
                    servicer.Like,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeResponse.SerializeToString,
            ),
            'List': grpc.unary_unary_rpc_method_handler(
                    servicer.List,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseListRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.SerializeToString,
            ),
            'Rate': grpc.unary_unary_rpc_method_handler(
                    servicer.Rate,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.RateRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.RateResponse.SerializeToString,
            ),
            'Retrieve': grpc.unary_unary_rpc_method_handler(
                    servicer.Retrieve,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.SerializeToString,
            ),
            'RetrieveComments': grpc.unary_unary_rpc_method_handler(
                    servicer.RetrieveComments,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveCommentsRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.SerializeToString,
            ),
            'RetrieveRates': grpc.unary_unary_rpc_method_handler(
                    servicer.RetrieveRates,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveRatesRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.SerializeToString,
            ),
            'UnLike': grpc.unary_unary_rpc_method_handler(
                    servicer.UnLike,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'darsman.course_lesson.ExpertCourseController', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class ExpertCourseController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def BulkRetrieve(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.ExpertCourseController/BulkRetrieve',
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseBulkRetrieveRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Comment(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.ExpertCourseController/Comment',
            course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def FetchMasterCourses(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.ExpertCourseController/FetchMasterCourses',
            course__lesson_dot_grpc_dot_course__lesson__pb2.MasterRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.MasterResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Like(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.ExpertCourseController/Like',
            course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def List(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.ExpertCourseController/List',
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseListRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Rate(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.ExpertCourseController/Rate',
            course__lesson_dot_grpc_dot_course__lesson__pb2.RateRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.RateResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Retrieve(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.ExpertCourseController/Retrieve',
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RetrieveComments(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.ExpertCourseController/RetrieveComments',
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveCommentsRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RetrieveRates(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.ExpertCourseController/RetrieveRates',
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveRatesRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UnLike(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.ExpertCourseController/UnLike',
            course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)


class GeneralControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Retrieve = channel.unary_unary(
                '/darsman.course_lesson.GeneralController/Retrieve',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.GeneralRetrieveRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.GeneralResponse.FromString,
                )


class GeneralControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Retrieve(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_GeneralControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Retrieve': grpc.unary_unary_rpc_method_handler(
                    servicer.Retrieve,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.GeneralRetrieveRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.GeneralResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'darsman.course_lesson.GeneralController', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class GeneralController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Retrieve(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.GeneralController/Retrieve',
            course__lesson_dot_grpc_dot_course__lesson__pb2.GeneralRetrieveRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.GeneralResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)


class LessonControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.BulkRetrieve = channel.unary_unary(
                '/darsman.course_lesson.LessonController/BulkRetrieve',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.LessonBulkRetrieveRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.Response.FromString,
                )
        self.Comment = channel.unary_unary(
                '/darsman.course_lesson.LessonController/Comment',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentResponse.FromString,
                )
        self.List = channel.unary_unary(
                '/darsman.course_lesson.LessonController/List',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.LessonListRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.LessonResponse.FromString,
                )
        self.Retrieve = channel.unary_unary(
                '/darsman.course_lesson.LessonController/Retrieve',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.LessonRetrieveRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.LessonResponse.FromString,
                )
        self.RetrieveComments = channel.unary_unary(
                '/darsman.course_lesson.LessonController/RetrieveComments',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.LessonRetrieveCommentsRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.LessonResponse.FromString,
                )


class LessonControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def BulkRetrieve(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Comment(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def List(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Retrieve(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RetrieveComments(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_LessonControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'BulkRetrieve': grpc.unary_unary_rpc_method_handler(
                    servicer.BulkRetrieve,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.LessonBulkRetrieveRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.Response.SerializeToString,
            ),
            'Comment': grpc.unary_unary_rpc_method_handler(
                    servicer.Comment,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentResponse.SerializeToString,
            ),
            'List': grpc.unary_unary_rpc_method_handler(
                    servicer.List,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.LessonListRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.LessonResponse.SerializeToString,
            ),
            'Retrieve': grpc.unary_unary_rpc_method_handler(
                    servicer.Retrieve,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.LessonRetrieveRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.LessonResponse.SerializeToString,
            ),
            'RetrieveComments': grpc.unary_unary_rpc_method_handler(
                    servicer.RetrieveComments,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.LessonRetrieveCommentsRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.LessonResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'darsman.course_lesson.LessonController', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class LessonController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def BulkRetrieve(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.LessonController/BulkRetrieve',
            course__lesson_dot_grpc_dot_course__lesson__pb2.LessonBulkRetrieveRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Comment(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.LessonController/Comment',
            course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def List(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.LessonController/List',
            course__lesson_dot_grpc_dot_course__lesson__pb2.LessonListRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.LessonResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Retrieve(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.LessonController/Retrieve',
            course__lesson_dot_grpc_dot_course__lesson__pb2.LessonRetrieveRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.LessonResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RetrieveComments(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.LessonController/RetrieveComments',
            course__lesson_dot_grpc_dot_course__lesson__pb2.LessonRetrieveCommentsRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.LessonResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)


class MasterCourseControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.BulkRetrieve = channel.unary_unary(
                '/darsman.course_lesson.MasterCourseController/BulkRetrieve',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseBulkRetrieveRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.Response.FromString,
                )
        self.Comment = channel.unary_unary(
                '/darsman.course_lesson.MasterCourseController/Comment',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentResponse.FromString,
                )
        self.CreateNotification = channel.unary_unary(
                '/darsman.course_lesson.MasterCourseController/CreateNotification',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.MasterCourseNotificationRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.NotificationResponse.FromString,
                )
        self.CreatePreRegistration = channel.unary_unary(
                '/darsman.course_lesson.MasterCourseController/CreatePreRegistration',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.MasterCoursePreRegistrationRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.PreRegistrationResponse.FromString,
                )
        self.FetchMasterCourses = channel.unary_unary(
                '/darsman.course_lesson.MasterCourseController/FetchMasterCourses',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.MasterRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.MasterResponse.FromString,
                )
        self.Like = channel.unary_unary(
                '/darsman.course_lesson.MasterCourseController/Like',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeResponse.FromString,
                )
        self.List = channel.unary_unary(
                '/darsman.course_lesson.MasterCourseController/List',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseListRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
                )
        self.Rate = channel.unary_unary(
                '/darsman.course_lesson.MasterCourseController/Rate',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.RateRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.RateResponse.FromString,
                )
        self.Retrieve = channel.unary_unary(
                '/darsman.course_lesson.MasterCourseController/Retrieve',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
                )
        self.RetrieveComments = channel.unary_unary(
                '/darsman.course_lesson.MasterCourseController/RetrieveComments',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveCommentsRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
                )
        self.RetrieveInstallmentTerm = channel.unary_unary(
                '/darsman.course_lesson.MasterCourseController/RetrieveInstallmentTerm',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.InstallmentTermRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.Response.FromString,
                )
        self.RetrieveRates = channel.unary_unary(
                '/darsman.course_lesson.MasterCourseController/RetrieveRates',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveRatesRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
                )
        self.RetrieveState = channel.unary_unary(
                '/darsman.course_lesson.MasterCourseController/RetrieveState',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.MasterCourseStateRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.StateResponse.FromString,
                )
        self.UnLike = channel.unary_unary(
                '/darsman.course_lesson.MasterCourseController/UnLike',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeResponse.FromString,
                )


class MasterCourseControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def BulkRetrieve(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Comment(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateNotification(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreatePreRegistration(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def FetchMasterCourses(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Like(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def List(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Rate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Retrieve(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RetrieveComments(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RetrieveInstallmentTerm(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RetrieveRates(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RetrieveState(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UnLike(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_MasterCourseControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'BulkRetrieve': grpc.unary_unary_rpc_method_handler(
                    servicer.BulkRetrieve,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseBulkRetrieveRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.Response.SerializeToString,
            ),
            'Comment': grpc.unary_unary_rpc_method_handler(
                    servicer.Comment,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentResponse.SerializeToString,
            ),
            'CreateNotification': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateNotification,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.MasterCourseNotificationRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.NotificationResponse.SerializeToString,
            ),
            'CreatePreRegistration': grpc.unary_unary_rpc_method_handler(
                    servicer.CreatePreRegistration,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.MasterCoursePreRegistrationRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.PreRegistrationResponse.SerializeToString,
            ),
            'FetchMasterCourses': grpc.unary_unary_rpc_method_handler(
                    servicer.FetchMasterCourses,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.MasterRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.MasterResponse.SerializeToString,
            ),
            'Like': grpc.unary_unary_rpc_method_handler(
                    servicer.Like,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeResponse.SerializeToString,
            ),
            'List': grpc.unary_unary_rpc_method_handler(
                    servicer.List,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseListRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.SerializeToString,
            ),
            'Rate': grpc.unary_unary_rpc_method_handler(
                    servicer.Rate,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.RateRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.RateResponse.SerializeToString,
            ),
            'Retrieve': grpc.unary_unary_rpc_method_handler(
                    servicer.Retrieve,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.SerializeToString,
            ),
            'RetrieveComments': grpc.unary_unary_rpc_method_handler(
                    servicer.RetrieveComments,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveCommentsRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.SerializeToString,
            ),
            'RetrieveInstallmentTerm': grpc.unary_unary_rpc_method_handler(
                    servicer.RetrieveInstallmentTerm,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.InstallmentTermRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.Response.SerializeToString,
            ),
            'RetrieveRates': grpc.unary_unary_rpc_method_handler(
                    servicer.RetrieveRates,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveRatesRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.SerializeToString,
            ),
            'RetrieveState': grpc.unary_unary_rpc_method_handler(
                    servicer.RetrieveState,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.MasterCourseStateRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.StateResponse.SerializeToString,
            ),
            'UnLike': grpc.unary_unary_rpc_method_handler(
                    servicer.UnLike,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'darsman.course_lesson.MasterCourseController', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class MasterCourseController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def BulkRetrieve(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.MasterCourseController/BulkRetrieve',
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseBulkRetrieveRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Comment(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.MasterCourseController/Comment',
            course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateNotification(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.MasterCourseController/CreateNotification',
            course__lesson_dot_grpc_dot_course__lesson__pb2.MasterCourseNotificationRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.NotificationResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreatePreRegistration(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.MasterCourseController/CreatePreRegistration',
            course__lesson_dot_grpc_dot_course__lesson__pb2.MasterCoursePreRegistrationRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.PreRegistrationResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def FetchMasterCourses(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.MasterCourseController/FetchMasterCourses',
            course__lesson_dot_grpc_dot_course__lesson__pb2.MasterRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.MasterResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Like(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.MasterCourseController/Like',
            course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def List(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.MasterCourseController/List',
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseListRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Rate(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.MasterCourseController/Rate',
            course__lesson_dot_grpc_dot_course__lesson__pb2.RateRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.RateResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Retrieve(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.MasterCourseController/Retrieve',
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RetrieveComments(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.MasterCourseController/RetrieveComments',
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveCommentsRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RetrieveInstallmentTerm(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.MasterCourseController/RetrieveInstallmentTerm',
            course__lesson_dot_grpc_dot_course__lesson__pb2.InstallmentTermRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RetrieveRates(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.MasterCourseController/RetrieveRates',
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveRatesRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RetrieveState(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.MasterCourseController/RetrieveState',
            course__lesson_dot_grpc_dot_course__lesson__pb2.MasterCourseStateRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.StateResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UnLike(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.MasterCourseController/UnLike',
            course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)


class PackageControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.BulkRetrieve = channel.unary_unary(
                '/darsman.course_lesson.PackageController/BulkRetrieve',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.PackageBulkRetrieveRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.Response.FromString,
                )
        self.List = channel.unary_unary(
                '/darsman.course_lesson.PackageController/List',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.PackageListRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.PackageResponse.FromString,
                )
        self.Retrieve = channel.unary_unary(
                '/darsman.course_lesson.PackageController/Retrieve',
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.PackageRetrieveRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.PackageResponse.FromString,
                )


class PackageControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def BulkRetrieve(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def List(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Retrieve(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_PackageControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'BulkRetrieve': grpc.unary_unary_rpc_method_handler(
                    servicer.BulkRetrieve,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.PackageBulkRetrieveRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.Response.SerializeToString,
            ),
            'List': grpc.unary_unary_rpc_method_handler(
                    servicer.List,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.PackageListRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.PackageResponse.SerializeToString,
            ),
            'Retrieve': grpc.unary_unary_rpc_method_handler(
                    servicer.Retrieve,
                    request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.PackageRetrieveRequest.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.PackageResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'darsman.course_lesson.PackageController', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class PackageController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def BulkRetrieve(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.PackageController/BulkRetrieve',
            course__lesson_dot_grpc_dot_course__lesson__pb2.PackageBulkRetrieveRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def List(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.PackageController/List',
            course__lesson_dot_grpc_dot_course__lesson__pb2.PackageListRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.PackageResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Retrieve(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.PackageController/Retrieve',
            course__lesson_dot_grpc_dot_course__lesson__pb2.PackageRetrieveRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.PackageResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)


class TopBannerControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.RetrieveCurrent = channel.unary_unary(
                '/darsman.course_lesson.TopBannerController/RetrieveCurrent',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TopBannerResponse.FromString,
                )


class TopBannerControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def RetrieveCurrent(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_TopBannerControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'RetrieveCurrent': grpc.unary_unary_rpc_method_handler(
                    servicer.RetrieveCurrent,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TopBannerResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'darsman.course_lesson.TopBannerController', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class TopBannerController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def RetrieveCurrent(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.course_lesson.TopBannerController/RetrieveCurrent',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.TopBannerResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



import asyncio
from typing import ClassVar, List, Optional
from uuid import UUID

from google.protobuf.json_format import MessageToDict
from google.protobuf.message import Message
from pydantic import (
    UUID4,
    BaseModel,
    StringConstraints,
    validate_call,
)
from typing_extensions import Annotated

from . import config
from .api_constants import (
    SUCCESS_CODE,
    InvokerUnits,
    RequestMessages,
    RPCMethodNames,
    Stubs,
)

IranianPhoneNumber = Annotated[
    str,
    StringConstraints(
        strip_whitespace=True, pattern=r"^(?:\+98|098|98|0098|)0?9\d{9}$"
    ),
]


class ValidationError(Exception):
    pass


class ResponseCodeError(Exception):
    pass


class RPCClient(BaseModel):
    _instances: ClassVar[dict] = {}
    stub_cls: ClassVar

    def __new__(cls, *args, **kwargs):
        # Ensure that each subclass has its own singleton instance
        if cls not in cls._instances:
            cls._instances[cls] = super().__new__(cls)
        return cls._instances[cls]

    @validate_call(config=dict(arbitrary_types_allowed=True))
    async def perform_rpc(
        self,
        *,
        rpc_method_name: str,
        request_message: Optional[Message] = None,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Perform the remote-precedure-call.

        Parameters:
            rpc_method_name: The name of the RPC method (e.g., `Retrieve`).
            request_message: The request message instance.
            timeout: RPC timeout.
            wait_for_ready: Wait for Channel to be ready in case is not.
        """
        if config.CHANNEL is None:
            await config.construct_channel()

        RPCMethod = getattr(self.stub_cls(config.CHANNEL), rpc_method_name, None)

        if RPCMethod:
            metadata = [("api-key", config.grpc_settings.API_KEY)]
            if request_id:
                metadata.append(("x-request-id", str(request_id)))
            response = await RPCMethod(
                request_message,
                metadata=metadata,
                timeout=config.grpc_settings.TIMEOUT if timeout is None else timeout,
                wait_for_ready=config.grpc_settings.WAIT_FOR_READY
                if wait_for_ready is None
                else wait_for_ready,
            )

            # Raise error if response is not successful
            if response.code != SUCCESS_CODE:
                raise ResponseCodeError(response.code)

            return MessageToDict(response, preserving_proto_field_name=True)
        else:
            raise ValidationError("Invalid RPC method name.")

    @validate_call(config=dict(arbitrary_types_allowed=True))
    async def perform_bulk_rpc(
        self,
        rpc_method_name: str,
        request_messages: List[Message],
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Perform the bulk remote-precedure-call.

        Parameters:
            rpc_method_name: The name of the RPC method (e.g., `Retrieve`).
            request_message: The request message instances for each call.
            timeout: RPC timeout of each call.
            wait_for_ready: Wait for Channel to be ready in case is not.
        """
        if config.CHANNEL is None:
            await config.construct_channel()

        RPCMethod = getattr(self.stub_cls(config.CHANNEL), rpc_method_name, None)

        if RPCMethod:
            metadata = [("api-key", config.grpc_settings.API_KEY)]
            if request_id:
                metadata.append(("x-request-id", str(request_id)))
            # Send all requests concurrently
            responses = await asyncio.gather(
                *[
                    RPCMethod(
                        request,
                        metadata=metadata,
                        timeout=config.grpc_settings.TIMEOUT
                        if timeout is None
                        else timeout,
                        wait_for_ready=config.grpc_settings.WAIT_FOR_READY
                        if wait_for_ready is None
                        else wait_for_ready,
                    )
                    for request in request_messages
                ]
            )

            # Process responses
            response_dicts = []
            for response in responses:
                if response.code != SUCCESS_CODE:
                    raise ResponseCodeError(response.code)
                response_dicts.append(
                    MessageToDict(response, preserving_proto_field_name=True)
                )

            return response_dicts
        else:
            raise ValidationError("Invalid RPC method name.")


class _RPCBase(RPCClient):
    """Base class for RPC operations, extending RPCClient for handling RPC calls."""

    fetch_one_request: ClassVar
    fetch_bulk_request: ClassVar
    fetch_all_request: ClassVar

    @validate_call
    async def fetch_one(
        self,
        identifier: Optional[str | UUID4] = None,
        slug: Optional[str] = None,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Fetch a single record from a database model."""
        request = self.fetch_one_request()
        if identifier:
            request.identifier = identifier  # Assign attributes
        if slug:
            request.slug = slug  # Assign attributes

        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.FETCH_ONE,
            request_message=request,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def fetch_bulk(
        self,
        identifiers: List[Optional[str | UUID4]],
        invoker_unit: InvokerUnits,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        request = self.fetch_bulk_request(
            identifier=[str(i) for i in identifiers], invoker_unit=invoker_unit
        )

        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.FETCH_BULK,
            request_message=request,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def fetch_one_for_search(
        self,
        identifier: Optional[str | UUID4] = None,
        slug: Optional[str] = None,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Fetch a single record from a database model with invoker_unit set to `search`."""
        request = self.fetch_one_request(invoker_unit=InvokerUnits.SEARCH)
        if identifier:
            request.identifier = identifier  # Assign attributes
        if slug:
            request.slug = slug  # Assign attributes

        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.FETCH_ONE,
            request_message=request,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def fetch_all(
        self,
        order_by: str,
        page_size: int,
        page_number: int,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Fetch all records from a database model."""
        request = self.fetch_all_request(
            order_by=order_by, page_size=page_size, page_number=page_number
        )

        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.FETCH_ALL,
            request_message=request,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def fetch_all_for_sitemap(
        self,
        order_by: str,
        page_size: int,
        page_number: int,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Fetch all records from a database model for sitemap use."""
        request = self.fetch_all_request(
            order_by=order_by,
            page_size=page_size,
            page_number=page_number,
            invoker_unit=InvokerUnits.PLACETAG_SITEMAP,
        )

        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.FETCH_ALL,
            request_message=request,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def fetch_comments(
        self,
        identifier: Optional[str | UUID4] = None,
        slug: Optional[str] = None,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Fetch a all `comments` of an item object."""
        # Assign attributes
        request = RequestMessages.FETCH_COURSE_COMMENTS()
        if identifier:
            request.identifier = str(identifier)
        if slug:
            request.slug = slug

        if not isinstance(self, Package):
            return await self.perform_rpc(
                rpc_method_name=RPCMethodNames.FETCH_COMMENTS,
                request_message=request,
                timeout=timeout,
                wait_for_ready=wait_for_ready,
            )
        raise ValidationError("package has no comments.")

    @validate_call
    async def comment(
        self,
        name: str,
        content: str,
        owner_identifier: Optional[str | UUID4] = None,
        owner_slug: Optional[str] = None,
        user_identifier: Optional[str | UUID4] = None,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Comment on a course/lesson."""
        # Assign attributes
        request = RequestMessages.COMMENT(name=name, content=content)

        # Since these are optional don't set if None
        if owner_identifier:
            request.owner_identifier = str(owner_identifier)
        if owner_slug:
            request.owner_slug = owner_slug
        if user_identifier:
            request.user_identifier = str(user_identifier)

        if not isinstance(self, Package):
            return await self.perform_rpc(
                rpc_method_name=RPCMethodNames.COMMENT,
                request_message=request,
                request_id=request_id,
                timeout=timeout,
                wait_for_ready=wait_for_ready,
            )
        raise ValidationError("Cannot comment on a package.")


class CourseBase(_RPCBase):
    """API: `fetch_one()` `fetch_bulk()` `fetch_one_for_search()` `fetch_all()`
    `fetch_all_for_sitemap()` `fetch_comments()` `comment()`, `fetch_rates()`."""

    fetch_one_request: ClassVar = RequestMessages.FETCH_COURSE
    fetch_bulk_request: ClassVar = RequestMessages.FETCH_BULK_COURSE
    fetch_all_request: ClassVar = RequestMessages.FETCH_ALL_COURSE

    @validate_call
    async def fetch_rates(
        self,
        identifier: Optional[str | UUID4] = None,
        slug: Optional[str] = None,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Fetch all `rates` of a course object."""
        # Assign attributes
        request = RequestMessages.FETCH_COURSE_RATES()
        if identifier:
            request.identifier = str(identifier)
        if slug:
            request.slug = slug

        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.FETCH_RATES,
            request_message=request,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )


class Course(CourseBase):
    """API: `fetch_one()` `fetch_bulk()` `fetch_one_for_search()` `fetch_all()`
    `fetch_all_for_sitemap()` `fetch_comments()` `comment()`, `fetch_rates()`
    `fetch_normals()` `fetch_roadmaps()` `like()` `unlike()` `rate()`.
    """

    stub_cls: ClassVar = Stubs.COURSE

    @validate_call
    async def fetch_normals(
        self,
        order_by: str,
        page_size: int,
        page_number: int,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Fetch all normal courses."""
        request = self.fetch_all_request(
            order_by=order_by,
            page_size=page_size,
            page_number=page_number,
            item_type="course",
        )
        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.FETCH_ALL,
            request_message=request,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def fetch_roadmaps(
        self,
        order_by: str,
        page_size: int,
        page_number: int,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Fetch all roadmap courses."""
        request = self.fetch_all_request(
            order_by=order_by,
            page_size=page_size,
            page_number=page_number,
            item_type="roadmap",
        )
        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.FETCH_ALL,
            request_message=request,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def like(
        self,
        course_identifier: Optional[str | UUID4] = None,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
        course_slug: Optional[str] = None,
    ):
        # Since these are optional don't set if None
        request = RequestMessages.LIKE()
        if course_identifier:
            request.owner_identifier = str(course_identifier)
        if course_slug:
            request.owner_slug = course_slug
        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.LIKE,
            request_message=request,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def unlike(
        self,
        course_identifier: Optional[str | UUID4] = None,
        course_slug: Optional[str] = None,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        # Since these are optional don't set if None
        request = RequestMessages.UNLIKE()
        if course_identifier:
            request.owner_identifier = str(course_identifier)
        if course_slug:
            request.owner_slug = course_slug
        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.UNLIKE,
            request_message=request,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def rate(
        self,
        user_identifier: str | UUID4,
        rate: float,
        course_identifier: Optional[str | UUID4] = None,
        course_slug: Optional[str] = None,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        # Assign attributes
        request = RequestMessages.RATE(user_identifier=str(user_identifier), rate=rate)

        # Since these are optional don't set if None
        if course_identifier:
            request.owner_identifier = str(course_identifier)
        if course_slug:
            request.owner_slug = course_slug

        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.RATE,
            request_message=request,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )


class ExpertCourse(Course):
    """API: `fetch_one()` `fetch_bulk()` `fetch_one_for_search()` `fetch_all()`
    `fetch_all_for_sitemap()` `fetch_comments()` `comment()`, `fetch_rates()`
    `fetch_normals()` `fetch_roadmaps()` `like()` `unlike()` `rate()`.
    """

    stub_cls: ClassVar = Stubs.EXPERT_COURSE


class MasterCourse(Course):
    """Methods: `fetch_one()`, `featch_all()`, `comment()`, `like()`, `unlike()`,
    `fetch_rates()`, `fetch_comments()`, `fetch_state()`, `create_notification()`,
    `create_pre_registration()`."""

    stub_cls: ClassVar = Stubs.MASTER_COURSE

    @validate_call
    async def fetch_state(
        self,
        identifier: Optional[str | UUID4],
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Fetch `state` info of a master course object."""
        # Assign attributes
        request = RequestMessages.FETCH_COURSE_STATE(identifier=str(identifier))
        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.FETCH_STATE,
            request_message=request,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def fetch_installment_term(
        self,
        identifier: Optional[str | UUID4],
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Fetch `installment_term` info of a master course object."""
        # Assign attributes
        request = RequestMessages.FETCH_COURSE_INSTALLMENT_TERM(
            identifier=str(identifier)
        )
        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.FETCH_INSTALLMENT_TERM,
            request_message=request,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def create_notification(
        self,
        identifier: Optional[str | UUID4],
        first_name: str,
        last_name: str,
        phone_number: IranianPhoneNumber,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """create a new notification for a master course object."""
        # Assign attributes
        request = RequestMessages.CREATE_NOTIFICATION(
            phone_number=phone_number,
            first_name=first_name,
            last_name=last_name,
            master_course=str(identifier),
        )
        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.CREATE_NOTIFICATION,
            request_message=request,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def create_pre_registration(
        self,
        identifier: Optional[str | UUID4],
        user_identifier: str,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """create a new pre_registration for a master course object."""
        # Assign attributes
        request = RequestMessages.CREATE_PRE_REGISTRATION(
            master_course=str(identifier), user_identifier=user_identifier
        )
        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.CREATE_PRE_REGISTRATION,
            request_message=request,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )


class Lesson(_RPCBase):
    """Configure the host and port of the course_lesson server.

    Parameters:
        host (str): The host address of the course_lesson server.
        port (int): The port number of the course_lesson server.
        api_key (str): The api_key of the course_lesson server.

    Methods: `fetch_one()`, `featch_all()`, `comment()`."""

    stub_cls: ClassVar = Stubs.LESSON
    fetch_one_request: ClassVar = RequestMessages.FETCH_LESSON
    fetch_bulk_request: ClassVar = RequestMessages.FETCH_BULK_LESSON
    fetch_all_request: ClassVar = RequestMessages.FETCH_ALL_LESSON


class Package(_RPCBase):
    """Configure the host and port of the course_lesson server.

    Parameters:
        host (str): The host address of the course_lesson server.
        port (int): The port number of the course_lesson server.
        api_key (str): The api_key of the course_lesson server.

    Methods: `fetch_one()`, `featch_all()`."""

    stub_cls: ClassVar = Stubs.PAKCAGE
    fetch_one_request: ClassVar = RequestMessages.FETCH_PACKAGE
    fetch_all_request: ClassVar = RequestMessages.FETCH_ALL_PACKAGE


class Category(RPCClient):
    """Configure the host and port of the course_lesson server.

    Parameters:
        host (str): The host address of the course_lesson server.
        port (int): The port number of the course_lesson server.
        api_key (str): The api_key of the course_lesson server.

    Methods: `hierarchical_fetch_all()`."""

    stub_cls: ClassVar = Stubs.CATEGORY

    async def hierarchical_fetch_all(
        self,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Fetch all records in a hierarchical structure."""
        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.HIERARCHICAL_FETCH_ALL,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def fetch_one(
        self,
        identifier: Optional[str | UUID4] = None,
        slug: Optional[str] = None,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Fetch a single record from a database model."""
        request = RequestMessages.FETCH_CATEGORY()
        if identifier:
            request.identifier = str(identifier)
        if slug:
            request.slug = slug
        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.FETCH_ONE,
            request_message=request,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )


class TopBanner(RPCClient):
    """Configure the host and port of the course_lesson server.

    Parameters:
        host (str): The host address of the course_lesson server.
        port (int): The port number of the course_lesson server.
        api_key (str): The api_key of the course_lesson server.

    Methods: `fetch_current()`."""

    stub_cls: ClassVar = Stubs.TOP_BANNER

    @validate_call
    async def fetch_current(
        self,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Fetch the current top banner object."""
        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.FETCH_CURRENT,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )


class General(RPCClient):
    """Configure the host and port of the course_lesson server.

    Parameters:
        host (str): The host address of the course_lesson server.
        port (int): The port number of the course_lesson server.
        api_key (str): The api_key of the course_lesson server.

    Methods: `fetch_one_for_asset()`, `fetch_one_for_basket()`, `fetch_one_for_asset_manager()`,
    `fetch_bulk_for_asset()`, `fetch_bulk_for_basket()`, `fetch_bulk_for_invoice()`."""

    stub_cls: ClassVar = Stubs.GENERAL

    @validate_call
    async def fetch_one_for_asset(
        self,
        identifier: Optional[str | UUID4],
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Fetch item by it's `identifeir` regardless of its type."""
        # Assign attributes
        request = RequestMessages.FETCH_ITEM(
            identifier=str(identifier), invoker_unit=InvokerUnits.ACCOUNT_ASSET
        )
        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.FETCH_ONE,
            request_message=request,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def fetch_one_for_basket(
        self,
        identifier: Optional[str | UUID4],
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Fetch item by it's `identifeir` regardless of its type."""
        # Assign attributes
        request = RequestMessages.FETCH_ITEM(
            identifier=str(identifier), invoker_unit=InvokerUnits.CORE_BASKET
        )
        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.FETCH_ONE,
            request_message=request,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def fetch_one_for_asset_manager(
        self,
        identifier: Optional[str | UUID4],
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Fetch item by it's `identifeir` regardless of its type."""
        # Assign attributes
        request = RequestMessages.FETCH_ITEM(
            identifier=str(identifier), invoker_unit=InvokerUnits.CORE_ASSET_MANAGEMENT
        )
        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.FETCH_ONE,
            request_message=request,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def fetch_bulk_for_asset(
        self,
        identifiers: List[Optional[str | UUID4]],
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Fetch multiple items by their identifiers."""
        # Prepare a list of request messages for all identifiers
        request_messages = [
            RequestMessages.FETCH_BULK_ITEM(
                identifier=str(identifier), invoker_unit=InvokerUnits.ACCOUNT_ASSET
            )
            for identifier in identifiers
        ]

        return await self.perform_bulk_rpc(
            rpc_method_name=RPCMethodNames.FETCH_ONE,
            request_messages=request_messages,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def fetch_bulk_for_basket(
        self,
        identifiers: List[Optional[str | UUID4]],
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Fetch multiple items by their identifiers."""
        # Prepare a list of request messages for all identifiers
        request_messages = [
            RequestMessages.FETCH_BULK_ITEM(
                identifier=str(identifier), invoker_unit=InvokerUnits.CORE_BASKET
            )
            for identifier in identifiers
        ]

        return await self.perform_bulk_rpc(
            rpc_method_name=RPCMethodNames.FETCH_ONE,
            request_messages=request_messages,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def fetch_bulk_for_invoice(
        self,
        identifiers: List[Optional[str | UUID4]],
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Fetch multiple items by their identifiers for use in invoice inquiry."""
        # Prepare a list of request messages for all identifiers
        request_messages = [
            RequestMessages.FETCH_BULK_ITEM(
                identifier=str(identifier), invoker_unit=InvokerUnits.INVOICE_INQUIRY
            )
            for identifier in identifiers
        ]

        return await self.perform_bulk_rpc(
            rpc_method_name=RPCMethodNames.FETCH_ONE,
            request_messages=request_messages,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )


class Consultant(RPCClient):
    """Configure the host and port of the course_lesson server.

    Parameters:
        host (str): The host address of the course_lesson server.
        port (int): The port number of the course_lesson server.
        api_key (str): The api_key of the course_lesson server.

    Methods: `create()`."""

    stub_cls: ClassVar = Stubs.CONSULTANT

    @validate_call
    async def create(
        self,
        phone_number: IranianPhoneNumber,
        full_name: Optional[str],
        item_type: Optional[str] = None,
        item_identifier: Optional[str | UUID4] = None,
        preferred_contact_time: Optional[str] = None,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """create a new Consultant request."""
        # Assign attributes
        request = RequestMessages.CREATE_CONSULTANT_REQUEST(
            phone_number=phone_number, full_name=full_name
        )
        if item_type:
            request.item_type = item_type
        if item_identifier:
            request.item_identifier = str(item_identifier)
        if preferred_contact_time:
            request.preferred_contact_time = preferred_contact_time
        return await self.perform_rpc(
            rpc_method_name=RPCMethodNames.CREATE,
            request_message=request,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )
